MicrosoftJacket = MicrosoftJacket or class()
MicrosoftJacket.debug = false
MicrosoftJacket._path = ModPath
MicrosoftJacket._data = {}
MicrosoftJacket._volume = 0.5
MicrosoftJacket._debug = false
MicrosoftJacket._current_state = "default"
MicrosoftJacket._states = {}
MicrosoftJacket._grudges_on_units = {}
MicrosoftJacket._warned_first = false
MicrosoftJacket._warned_second = false
MicrosoftJacket._warned_third = false
MicrosoftJacket._sound_play_backs = {}
function MicrosoftJacket:init(is_menu)
	self:get_custom_clips()
	self.is_menu = is_menu
end

function MicrosoftJacket:sounds_directory(folder, filename)
	return self._path.."assets/sounds"..(folder and "/"..folder.."/" or "")..filename..".ogg"
end


Hooks:Add("LocalizationManagerPostInit", "LocalizationManagerPostInit_MicrosoftJacket", function( loc )
	loc:load_localization_file(MicrosoftJacket._path .. "assets/loc/en.txt")
end)
function MicrosoftJacket:_load_json(file_path)
	local file = io.open( file_path, "r" )
	if file then
		local file_content = file:read("*all")
		file:close()
		return json.decode(file_content)
	end
end
MicrosoftJacket._subtitles_data = MicrosoftJacket:_load_json(MicrosoftJacket._path.."assets/subtitles.json")

function MicrosoftJacket:create_sound_play_back_from_id(ids, custom_table)
	if type(ids) == "string" then
		ids = {ids}
	end
	local sound_play_back = SoundPlayBack:new(ids)	
	table.insert(self._sound_play_backs, sound_play_back)	
	return sound_play_back
end

function MicrosoftJacket:add_sounds_to_play_back(sound_play_back, datas, playback_chance)
	for _, data in pairs(datas) do
		if data.slot then
			for i,v in pairs(data.file_data) do
				if file.FileExists(v.file) then
					sound_play_back:add_to_sound_category(data.slot, v.file, v.chance, v.can_repeat, v.string_id, v.volume, v.markers)
				else			
					log("[MicrosoftJacket:add_sounds_to_play_back] Filnename "..tostring(v.file).." does not exist!!")
				end
			end
			if data.slot_chance then
				sound_play_back:add_slot_chance(data.slot, data.slot_chance)
			end
			if data.slot_delay then
				sound_play_back:add_slot_delay(data.slot, data.slot_delay)
			end
			if data.rare_chance then
				sound_play_back:set_rare_chance(data.slot, data.rare_chance)
			end
		end
	end
	if playback_chance then
		sound_play_back:set_playback_chance(playback_chance)
	end
end

function MicrosoftJacket:switch_state(state)
	self._current_state = state
end


function MicrosoftJacket:add_play_back_to_state(state, playback)
	self._states[state] = self._states[state] or {}
	self._states[state].playbacks = self._states[state].playbacks or {}
	table.insert(self._states[state].playbacks, playback)
end

function MicrosoftJacket:use_default_attributes(state, from_state)
	if not self._states[state] then
		return
	end
	self._states[state].use_attributes = from_state
end

function MicrosoftJacket:create_and_add_to_playback_from_ids(ids, datas, states, playback_chance)
	local playback = self:create_sound_play_back_from_id(ids)
	self:add_sounds_to_play_back(playback, datas, playback_chance)
	playback:flush()
	
	
	if not states then
		states = {"default"}
	elseif type(states) == "string" then
		states = {states}
	end
	

	
	for index, state in pairs(states) do
		self:add_play_back_to_state(state, playback)
	end
end




function MicrosoftJacket:state_exists(state)
	return self._states[state] and true or false
end

function MicrosoftJacket:_get_sound_playback_from_id(sound, state, skip_use_attributes)
	if not blt or not blt.xaudio then
		return
	end
	state = state or self._current_state
	if state and self._states[state] and self._states[state].playbacks then
		local outcome = math.random()
		local clips = {}
		for i, snd_play_back in pairs(self._states[state].playbacks or {}) do
			if snd_play_back and snd_play_back:contains_id(sound) then
				if snd_play_back:playback_rng(outcome) then
					table.insert(clips, snd_play_back)
				end
			end
		end		
		if next(clips) then
			return clips[math.random(#clips)]
		end
		if not skip_use_attributes and self._states[state].use_attributes then
			return self:_get_sound_playback_from_id(sound, self._states[state].use_attributes, true)
		end
	end
	return nil
end

dofile(ModPath.."Menu/MasterChief_clips.lua")
dofile(ModPath.."Menu/MasterChief_heisterclips.lua")
dofile(ModPath.."xaudio/XAudioUnitSource2.lua")

function MicrosoftJacket:_buffer_close()
	if self._sam_buffer then
		self._sam_buffer:close()
		self._sam_buffer = nil
	end
end
function MicrosoftJacket:mod_directory()
	return self._path
end

function MicrosoftJacket:update_sounds(t, dt, paused)
	for i, snd_play_back in pairs(self._sound_play_backs) do
		if snd_play_back then
			snd_play_back:update_sound(t, paused)
		end
	end
end

function MicrosoftJacket:on_downed(unit)

	if not unit then
		return
	end
	
	self._grudges_on_units[unit:key()] = self._grudges_on_units[unit:key()] or {}
	
	local grudge = self._grudges_on_units[unit:key()]
	
	
	local t = TimerManager:game():time()
	if grudge._downed_mark_t and t - grudge._downed_mark_t < 30 then 
		grudge._downed_counter = (grudge._downed_counter or 0) + 1
	else
		grudge._downed_counter = nil
	end
	
	grudge._downed_mark_t = t	
	if grudge._downed_counter == math.random(4,6) then
		if managers.player:player_unit() == unit then
			if managers.dialog and managers.groupai and managers.groupai:state():bain_state() then
				managers.dialog:queue_narrator_dialog("l01", {delay = 2})
			end
		end
		grudge._downed_counter = nil
	end
end

function MicrosoftJacket:stop_dialog()
	if managers and managers.dialog and managers.dialog._current_dialog then
		if managers.subtitle then
			managers.subtitle:set_visible(false)
			managers.subtitle:set_enabled(false)
		end
		managers.dialog:finished()
	end
end

function MicrosoftJacket:stop_cue()
	for id, snd_play_back in pairs(self._sound_play_backs) do
		if snd_play_back then
			snd_play_back:destroy_sound_source()
		end
	end
end


function MicrosoftJacket:play_subtitle(string_id, buffer)
	local duration = tonumber(buffer:get_length())
	if DramaExt then
		DramaExt:play_subtitle(string_id, duration)
	end
end



function MicrosoftJacket:_get_heister_sound_playback_from_id(sound)
	if not self._current_wave_flag then
		return false
	end
	for i, snd_play_back in pairs(self._wave_flags[self._current_wave_flag]) do
		if snd_play_back and snd_play_back:contains_id(sound) then
			return snd_play_back
		end
	end
end




function MicrosoftJacket:play_custom_sound(sound)
	if XAudio then
		local snd_play_back = self:_get_sound_playback_from_id(sound)	
		if snd_play_back then
			if not snd_play_back:playing() then			
				local clbk = callback(MicrosoftJacket, MicrosoftJacket, "stop_dialog")
				snd_play_back:_play_sounds({
					type = "dialog",
					end_clbk = clbk
				})
			end
			return true
		end
	end
	return false
end